﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Data;

namespace WpfApp2
{
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            var source = new ObservableCollection<RowModel>();
            for (int i = 0; i < 10; i++)
            {
                source.Add(new RowModel()
                {
                    Id = i,
                    FredBar = new Fred()
                    {
                        Date = DateTime.Now.AddDays(i),
                        Test = "gosho " + i,
                        Title = "Fred " + i,
                        Value = i * 10,
                    }
                });
            }

            this.DataContext = source;
        }

        private void GridView_Sorting(object sender, Telerik.Windows.Controls.GridViewSortingEventArgs e)
        {
            if (e.Column.UniqueName != "FredBar.Date" && e.Column.UniqueName != "FredBar.Test")
            {
                return;
            }

            IEnumerable<RowModel> items = e.DataControl.ItemsSource as IEnumerable<RowModel>;
            
            if (items == null)
            {
                e.Cancel = true;
                return;
            }
            
            if (e.OldSortingState == SortingState.None)
            {
                e.NewSortingState = SortingState.Ascending;
                items = SortAscendingByColumnUniqueName(e.Column.UniqueName, items);
            }            
            else if (e.OldSortingState == SortingState.Ascending)
            {
                e.NewSortingState = SortingState.Descending;
                items = SortDescendingByColumnUniqueName(e.Column.UniqueName, items);
            }
            else
            {
                e.NewSortingState = SortingState.None;
                items = items.OrderBy(employee => employee.Id);
            }

            e.DataControl.SortDescriptors.Clear();
            e.DataControl.ItemsSource = items;
            e.Cancel = true;
        }

        public IEnumerable<RowModel> SortAscendingByColumnUniqueName(string name, IEnumerable<RowModel> items)
        {
            switch (name)
            {
                case "FredBar.Date":
                    return items.OrderBy(x => ((Fred)x.FredBar).Date);
                case "FredBar.Test":
                    return items.OrderBy(x => ((Fred)x.FredBar).Test);
            }
            return items;
        }

        public IEnumerable<RowModel> SortDescendingByColumnUniqueName(string name, IEnumerable<RowModel> items)
        {
            switch (name)
            {
                case "FredBar.Date":
                    return items.OrderByDescending(x => ((Fred)x.FredBar).Date);
                case "FredBar.Test":
                    return items.OrderByDescending(x => ((Fred)x.FredBar).Test);
            }
            return items;
        }
    }

    public class RowModel
    {
        public int Id { get; set; }
        public Bar FredBar { get; set; }
    }

    public class Fred : Bar
    {
        public DateTime Date { get; set; }
        public string Test { get; set; }
    }

    public abstract class Bar
    {
        public string Title { get; set; }
        public double Value { get; set; }        
    }
}
